﻿using Arction.Wpf.Charting;
using Arction.Wpf.Charting.SeriesXY;
using Arction.Wpf.Charting.Views.ViewXY;
using DevExpress.Mvvm.Native;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media;

namespace WpfTaskChart
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window, INotifyPropertyChanged
    {

        public bool IsBusy
        {
            get => _isBusy;
            private set
            {
                if (_isBusy != value)
                {
                    _isBusy = value;
                    PropertyChanged?.Invoke(this, new PropertyChangedEventArgs("IsBusy")); ;
                }
            }
        }

        private LightningChartUltimate _chartMain;
        private LightningChartUltimate _chartSecond;
        private List<LightningChartUltimate> _charts;
        private bool _isBusy;

        public event PropertyChangedEventHandler PropertyChanged;

        public MainWindow()
        {
            LightningChartUltimate.SetDeploymentKey("your license");
            _chartMain = null;
            _chartSecond = null;
            InitializeComponent();
            CreateMainChart();
            CreateSecondChart();
            _charts = new List<LightningChartUltimate> {_chartMain, _chartSecond};
            this.DataContext = this;
        }

        private void CreateMainChart()
        {
            Chart1.Children.Clear();
            if (_chartMain != null)
            {
                // If a chart is already created, dispose it.
                _chartMain.Dispose();
                _chartMain = null;
            }
            _chartMain = new LightningChartUltimate
            {
                ChartName = "Test"
            };
            _chartMain.BeginUpdate();
            _chartMain.Title.MoveByMouse = false;
            _chartMain.Title.MouseHighlight = MouseOverHighlight.None;
            _chartMain.Title.Text = "Squib1";
            _chartMain.Title.Color = Colors.White;
            _chartMain.ColorTheme = ColorTheme.Dark;

            _chartMain.ChartBackground.Color = Colors.Black;
            _chartMain.ChartBackground.GradientColor = Colors.Black;
            _chartMain.ViewXY.GraphBackground.Color = Colors.Black;
            _chartMain.ViewXY.GraphBackground.GradientColor = Colors.Black;

            _chartMain.ViewXY.DropOldSeriesData = false;
            _chartMain.ChartRenderOptions.LineAAType2D = LineAntiAliasingType.QLAA;
            //Y-axis are stacked and has common x-axis drawn
            _chartMain.ViewXY.AxisLayout.YAxesLayout = YAxesLayout.Layered;
            _chartMain.ViewXY.AxisLayout.YAxisAutoPlacement = YAxisAutoPlacement.LeftThenRight;
            _chartMain.ViewXY.AxisLayout.AutoAdjustMargins = true;
            _chartMain.ViewXY.AxisLayout.XAxisAutoPlacement = XAxisAutoPlacement.Off;

            _chartMain.AfterRendering += _chartMain_AfterRendering;
            _chartMain.BeforeRendering += _chartMain_BeforeRendering;

            PointLineSeries series = new PointLineSeries(_chartMain.ViewXY, _chartMain.ViewXY.XAxes[0], _chartMain.ViewXY.YAxes[0]);
            series.AddPoints(GeneratePoints(1), true);
            _chartMain.ViewXY.PointLineSeries.Add(series);
            
            var mainCursor = new LineSeriesCursor(_chartMain.ViewXY, _chartMain.ViewXY.XAxes[0])
            {
                Tag = "Cursor #1",
                LineStyle =
                {
                    Width = 2f,
                    Color = Colors.Red
                },
                SnapToPoints = true,
                Visible = true,
                ValueAtXAxis = 5,
                TrackPoint =
                {
                    Color1 = Colors.White
                }
            };

            _chartMain.ViewXY.LineSeriesCursors.Add(mainCursor);

            _chartMain.ViewXY.ZoomToFit();

            _chartMain.EndUpdate();

            Chart1.Children.Add(_chartMain);
        }

        private void CreateSecondChart()
        {
            Chart2.Children.Clear();
            if (_chartSecond != null)
            {
                // If a chart is already created, dispose it.
                _chartSecond.Dispose();
                _chartSecond = null;
            }
            _chartSecond = new LightningChartUltimate
            {
                ChartName = "Test"
            };
            _chartSecond.BeginUpdate();
            _chartSecond.Title.MoveByMouse = false;
            _chartSecond.Title.MouseHighlight = MouseOverHighlight.None;
            _chartSecond.Title.Text = "Squib 2";
            _chartSecond.Title.Color = Colors.White;
            _chartSecond.ColorTheme = ColorTheme.Dark;

            _chartSecond.ChartBackground.Color = Colors.Black;
            _chartSecond.ChartBackground.GradientColor = Colors.Black;
            _chartSecond.ViewXY.GraphBackground.Color = Colors.Black;
            _chartSecond.ViewXY.GraphBackground.GradientColor = Colors.Black;

            _chartSecond.ViewXY.DropOldSeriesData = false;
            _chartSecond.ChartRenderOptions.LineAAType2D = LineAntiAliasingType.QLAA;
            //Y-axis are stacked and has common x-axis drawn
            _chartSecond.ViewXY.AxisLayout.YAxesLayout = YAxesLayout.Layered;
            _chartSecond.ViewXY.AxisLayout.YAxisAutoPlacement = YAxisAutoPlacement.LeftThenRight;
            _chartSecond.ViewXY.AxisLayout.AutoAdjustMargins = true;
            _chartSecond.ViewXY.AxisLayout.XAxisAutoPlacement = XAxisAutoPlacement.Off;

            _chartSecond.AfterRendering += _chartMain_AfterRendering;
            _chartSecond.BeforeRendering += _chartMain_BeforeRendering;

            PointLineSeries series = new PointLineSeries(_chartSecond.ViewXY, _chartSecond.ViewXY.XAxes[0], _chartSecond.ViewXY.YAxes[0]);
            series.AddPoints(GeneratePoints(2), true);
            _chartSecond.ViewXY.PointLineSeries.Add(series);
            var mainCursor = new LineSeriesCursor(_chartSecond.ViewXY, _chartSecond.ViewXY.XAxes[0])
            {
                Tag = "Cursor #1",
                LineStyle =
                {
                    Width = 2f,
                    Color = Colors.Red
                },
                SnapToPoints = true,
                Visible = true,
                ValueAtXAxis = 3,
                TrackPoint =
                {
                    Color1 = Colors.White
                }
            };

            _chartSecond.ViewXY.LineSeriesCursors.Add(mainCursor);

            _chartSecond.ViewXY.ZoomToFit();
            

            _chartSecond.EndUpdate();

            Chart2.Children.Add(_chartSecond);
        }


        private SeriesPoint[] GeneratePoints(int index)
        {
            var rv = new SeriesPoint[30];
            var idx = 0;
            if(index == 1)
            { 
                for (int i = 0; i < 30; i++)
                {
                    if (i >= 0 && i <= 5)
                        rv[idx++] = new SeriesPoint(i, 0.1);
                    if (i > 5 && i <= 10)
                        rv[idx++] = new SeriesPoint(i, 8.99);
                    if (i > 10)
                        rv[idx++] = new SeriesPoint(i, 0.99);
                }
            }
            else
            {
                for (int i = 0; i < 30; i++)
                {
                    if (i >= 0 && i <= 10)
                        rv[idx++] = new SeriesPoint(i, 0.1);
                    if (i > 10 && i <= 15)
                        rv[idx++] = new SeriesPoint(i, 8.99);
                    if (i > 15)
                        rv[idx++] = new SeriesPoint(i, 0.99);
                }
            }
            return rv;
        }

        private void _chartMain_BeforeRendering(object sender, BeforeRenderingEventArgs e)
        {
           
        }

        private void _chartMain_AfterRendering(object sender, AfterRenderingEventArgs e)
        {
           
        }
        
        /// <summary>
        /// Method for logging messages
        /// </summary>
        /// <param name="msg"></param>
        private static void LOG(string msg)
        {
            System.Diagnostics.Trace.WriteLine(msg);
        }

        private async void Run_OnClick(object sender, RoutedEventArgs e)
        {
            IsBusy = true;
            Stopwatch sw = new Stopwatch();
            sw.Start();
            simpleButton.Content = simpleButton.Content.ToString().Equals("Start") ? "Stop" : "Start";
            ProgressBarEdit.Visibility = Visibility.Visible;
            Dictionary<string, List<Bitmap>> collection = new Dictionary<string, List<Bitmap>>();
            await Task.Factory.StartNew(() =>
            {
                Parallel.ForEach(_charts,  ch =>
                {
                    var idx = _charts.IndexOf(ch);
                    LCRunner LC = new LCRunner(LOG, ch.Title.Text, idx + 1, -10, 300, 800, 600, 1, ImageFormat.Png);
                    var task = LC.GenerateImages();
                    collection.Add(ch.Title.Text, task);
                    LC.Dispose();
                    LC = null;
                });
              
            }).ContinueWith( taskInfo =>  
            {
                Parallel.ForEach(collection, col =>
                {
                    Parallel.ForEach(col.Value, b =>
                    {
                        var indx = col.Value.IndexOf(b) + 1;
                        b.Save($"{col.Key}_{indx}.png", ImageFormat.Png);
                        LOG($"{col.Key}_{indx}.png size is {b.Size.ToString()}");

                    });
                });


            });
            sw.Stop();
           
            LOG($"Processed using Run method in {sw.ElapsedMilliseconds} ms.");
            simpleButton.Content = simpleButton.Content.ToString().Equals("Start") ? "Stop" : "Start";
            ProgressBarEdit.Visibility = Visibility.Collapsed;
            IsBusy = false;
        }

        private List<byte[]> CreateFrames(Dictionary<string, List<Bitmap>> collection)
        {
            List<byte[]> clFrames = new List<byte[]>();
            collection.Values.ForEach(x =>
            {
                x.ForEach(y =>
                {
                    MemoryStream ms = new MemoryStream();
                    // Save to memory using the Jpeg format
                    y.Save(ms, ImageFormat.Jpeg);
                    // read to end
                    byte[] bmpBytes = ms.GetBuffer();
                    y.Dispose();
                    ms.Close();
                    clFrames.Add(bmpBytes);
                });
            });
            return clFrames;
        }
    }
}
